@@ -1,15 +1,16 @@
 package e2e_test
 
 import (
+	"strconv"
 	"strings"
 	"time"
 
-	"github.com/containers/common/pkg/util"
-	"github.com/containers/podman/v4/pkg/domain/entities"
+	"github.com/containers/podman/v5/pkg/domain/entities"
 	jsoniter "github.com/json-iterator/go"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gexec"
+	"golang.org/x/exp/slices"
 )
 
 var _ = Describe("podman machine list", func() {
@@ -33,7 +34,7 @@ var _ = Describe("podman machine list", func() {
 		Expect(firstList.outputToStringSlice()).To(HaveLen(1)) // just the header
 
 		i := new(initMachine)
-		session, err := mb.setCmd(i.withImagePath(mb.imagePath)).run()
+		session, err := mb.setCmd(i.withImage(mb.imagePath)).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(session).To(Exit(0))
 
@@ -60,11 +61,11 @@ var _ = Describe("podman machine list", func() {
 		Expect(noheaderSession.outputToStringSlice()).To(BeEmpty())
 
 		i := new(initMachine)
-		session, err := mb.setName(name1).setCmd(i.withImagePath(mb.imagePath)).run()
+		session, err := mb.setName(name1).setCmd(i.withImage(mb.imagePath)).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(session).To(Exit(0))
 
-		session2, err := mb.setName(name2).setCmd(i.withImagePath(mb.imagePath)).run()
+		session2, err := mb.setName(name2).setCmd(i.withImage(mb.imagePath)).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(session2).To(Exit(0))
 
@@ -75,13 +76,14 @@ var _ = Describe("podman machine list", func() {
 
 		listNames := secondList.outputToStringSlice()
 		stripAsterisk(listNames)
-		Expect(util.StringInSlice(name1, listNames)).To(BeTrue())
-		Expect(util.StringInSlice(name2, listNames)).To(BeTrue())
+		Expect(slices.Contains(listNames, name1)).To(BeTrue())
+		Expect(slices.Contains(listNames, name2)).To(BeTrue())
 	})
 
 	It("list machine: check if running while starting", func() {
+		skipIfWSL("the below logic does not work on WSL.  #20978")
 		i := new(initMachine)
-		session, err := mb.setCmd(i.withImagePath(mb.imagePath)).run()
+		session, err := mb.setCmd(i.withImage(mb.imagePath)).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(session).To(Exit(0))
 
@@ -91,10 +93,14 @@ var _ = Describe("podman machine list", func() {
 		Expect(listSession).To(Exit(0))
 		Expect(listSession.outputToString()).To(Equal("Never"))
 
+		// The logic in this test stanza is seemingly invalid on WSL.
+		// issue #20978 reflects this change
 		s := new(startMachine)
 		startSession, err := mb.setCmd(s).runWithoutWait()
 		Expect(err).ToNot(HaveOccurred())
-		for i := 0; i < 30; i++ {
+		wait := 3
+		retries := (int)(mb.timeout/time.Second) / wait
+		for i := 0; i < retries; i++ {
 			listSession, err := mb.setCmd(l).run()
 			Expect(listSession).To(Exit(0))
 			Expect(err).ToNot(HaveOccurred())
@@ -103,7 +109,7 @@ var _ = Describe("podman machine list", func() {
 			} else {
 				break
 			}
-			time.Sleep(3 * time.Second)
+			time.Sleep(time.Duration(wait) * time.Second)
 		}
 		Expect(startSession).To(Exit(0))
 		listSession, err = mb.setCmd(l).run()
@@ -118,7 +124,7 @@ var _ = Describe("podman machine list", func() {
 		name1 := randomString()
 
 		i := new(initMachine)
-		session, err := mb.setName(name1).setCmd(i.withImagePath(mb.imagePath)).run()
+		session, err := mb.setName(name1).setCmd(i.withImage(mb.imagePath)).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(session).To(Exit(0))
 
@@ -131,7 +137,7 @@ var _ = Describe("podman machine list", func() {
 
 		listNames := listSession.outputToStringSlice()
 		stripAsterisk(listNames)
-		Expect(util.StringInSlice(name1, listNames)).To(BeTrue())
+		Expect(slices.Contains(listNames, name1)).To(BeTrue())
 
 		// --format json
 		list2 := new(listMachine)
@@ -153,6 +159,40 @@ var _ = Describe("podman machine list", func() {
 		listNames3 := listSession3.outputToStringSlice()
 		Expect(listNames3).To(HaveLen(2))
 	})
+	It("list machine in machine-readable byte format", func() {
+		i := new(initMachine)
+		session, err := mb.setCmd(i.withImage(mb.imagePath)).run()
+		Expect(err).ToNot(HaveOccurred())
+		Expect(session).To(Exit(0))
+
+		list := new(listMachine)
+		list = list.withFormat(("json"))
+		listSession, err := mb.setCmd(list).run()
+		Expect(err).NotTo(HaveOccurred())
+		var listResponse []*entities.ListReporter
+		err = jsoniter.Unmarshal(listSession.Bytes(), &listResponse)
+		Expect(err).NotTo(HaveOccurred())
+		for _, reporter := range listResponse {
+			memory, err := strconv.Atoi(reporter.Memory)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(memory).To(BeNumerically(">", 2000000000)) // 2GiB
+			diskSize, err := strconv.Atoi(reporter.DiskSize)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(diskSize).To(BeNumerically(">", 11000000000)) // 11GiB
+		}
+	})
+	It("list machine in human-readable format", func() {
+		i := new(initMachine)
+		session, err := mb.setCmd(i.withImage(mb.imagePath)).run()
+		Expect(err).ToNot(HaveOccurred())
+		Expect(session).To(Exit(0))
+
+		list := new(listMachine)
+		listSession, err := mb.setCmd(list.withFormat("{{.Memory}} {{.DiskSize}}")).run()
+		Expect(err).NotTo(HaveOccurred())
+		Expect(listSession).To(Exit(0))
+		Expect(listSession.outputToString()).To(Equal("2GiB 11GiB"))
+	})
 })
 
 func stripAsterisk(sl []string) {
