@@ -1,7 +1,10 @@
 package machine
 
 import (
-	"strings"
+	"crypto/sha256"
+	"encoding/hex"
+
+	"github.com/containers/podman/v5/pkg/machine/vmconfigs"
 )
 
 type Volume interface {
@@ -27,18 +30,17 @@ func (v VirtIoFs) Kind() string {
 	return string(VirtIOFsVk)
 }
 
-// unitName is the fq path where /'s are replaced with -'s
-func (v VirtIoFs) unitName() string {
-	// delete the leading -
-	unit := strings.ReplaceAll(v.Target, "/", "-")
-	if strings.HasPrefix(unit, "-") {
-		return unit[1:]
-	}
-	return unit
+// generateTag generates a tag for VirtIOFs mounts.
+// AppleHV requires tags to be 36 bytes or fewer.
+// SHA256 the path, then truncate to 36 bytes
+func (v VirtIoFs) generateTag() string {
+	sum := sha256.Sum256([]byte(v.Target))
+	stringSum := hex.EncodeToString(sum[:])
+	return stringSum[:36]
 }
 
-func (v VirtIoFs) ToMount() Mount {
-	return Mount{
+func (v VirtIoFs) ToMount() vmconfigs.Mount {
+	return vmconfigs.Mount{
 		ReadOnly: v.ReadOnly,
 		Source:   v.Source,
 		Tag:      v.Tag,
@@ -56,6 +58,16 @@ func NewVirtIoFsMount(src, target string, readOnly bool) VirtIoFs {
 		Source:   src,
 		Target:   target,
 	}
-	vfs.Tag = vfs.unitName()
+	vfs.Tag = vfs.generateTag()
 	return vfs
 }
+
+func MountToVirtIOFs(mnt *vmconfigs.Mount) VirtIoFs {
+	return VirtIoFs{
+		VolumeKind: VirtIOFsVk,
+		ReadOnly:   mnt.ReadOnly,
+		Source:     mnt.Source,
+		Tag:        mnt.Tag,
+		Target:     mnt.Target,
+	}
+}
