@@ -6,7 +6,7 @@ import (
 	"os/exec"
 	"time"
 
-	. "github.com/containers/podman/v4/test/utils"
+	. "github.com/containers/podman/v5/test/utils"
 	"github.com/containers/storage/pkg/stringid"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
@@ -536,7 +536,7 @@ var _ = Describe("Podman logs", func() {
 
 	It("using journald for container with container tag", func() {
 		SkipIfJournaldUnavailable()
-		logc := podmanTest.Podman([]string{"run", "--log-driver", "journald", "--log-opt=tag={{.ImageName}}", "-d", ALPINE, "sh", "-c", "echo podman; sleep 0.1; echo podman; sleep 0.1; echo podman"})
+		logc := podmanTest.Podman([]string{"run", "--log-driver", "journald", "--log-opt=tag={{.ImageName}},withcomma", "-d", ALPINE, "sh", "-c", "echo podman; sleep 0.1; echo podman; sleep 0.1; echo podman"})
 		logc.WaitWithDefaultTimeout()
 		Expect(logc).To(ExitCleanly())
 		cid := logc.OutputToString()
@@ -549,7 +549,7 @@ var _ = Describe("Podman logs", func() {
 			cmd := exec.Command("journalctl", "--no-pager", "-o", "json", "--output-fields=CONTAINER_TAG", fmt.Sprintf("CONTAINER_ID_FULL=%s", cid))
 			out, err := cmd.CombinedOutput()
 			g.Expect(err).ToNot(HaveOccurred())
-			g.Expect(string(out)).To(ContainSubstring("alpine"))
+			g.Expect(string(out)).To(ContainSubstring(ALPINE + ",withcomma"))
 		}).Should(Succeed())
 	})
 
@@ -596,12 +596,13 @@ var _ = Describe("Podman logs", func() {
 		logc := podmanTest.Podman([]string{"run", "--log-driver", "journald", "--log-opt", "tag=äöüß", ALPINE, "echo", "podman"})
 		logc.WaitWithDefaultTimeout()
 		Expect(logc).To(Exit(126))
+		// FIXME-2023-09-26: conmon <2.1.8 logs to stdout; clean this up once >=2.1.8 is universal
+		errmsg := logc.ErrorToString() + logc.OutputToString()
 		if !IsRemote() {
 			// Error is only seen on local client
-			// Why does conmon log this to stdout? This must be fixed after https://github.com/containers/conmon/pull/447.
-			Expect(logc.OutputToString()).To(Equal("conmon: option parsing failed: Invalid byte sequence in conversion input"))
+			Expect(errmsg).To(ContainSubstring("conmon: option parsing failed: Invalid byte sequence in conversion input"))
 		}
-		Expect(logc.ErrorToString()).To(ContainSubstring("conmon failed: exit status 1"))
+		Expect(errmsg).To(ContainSubstring("conmon failed: exit status 1"))
 	})
 
 	It("podman logs with non ASCII log tag succeeds with proper env", func() {
